from fastapi import APIRouter, Depends, HTTPException
from ..services.question_service import QuestionService
from ..models.question import Question
from ..models.question_bank_permission import QuestionBankPermission
from typing import List

router = APIRouter(prefix="/api/questions", tags=["questions"])


@router.post("/", response_model=Question)
async def create_question(question_data: dict):
    """创建题目"""
    try:
        return QuestionService().create_question(question_data)
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.get("/{question_id}", response_model=Question)
async def get_question(question_id: int):
    """获取题目详情"""
    try:
        return QuestionService().get_question(question_id)
    except Exception as e:
        raise HTTPException(status_code=404, detail=str(e))


@router.put("/{question_id}", response_model=Question)
async def update_question(question_id: int, update_data: dict):
    """更新题目"""
    try:
        return QuestionService().update_question(question_id, update_data)
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.delete("/{question_id}")
async def delete_question(question_id: int):
    """删除题目"""
    try:
        QuestionService().delete_question(question_id)
        return {"message": "Question deleted successfully"}
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.get("/search/", response_model=List[Question])
async def search_questions(q: str, limit: int = 100):
    """搜索题目"""
    try:
        return QuestionService().search_questions(q, limit)
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.get("/bank/{bank_id}", response_model=List[Question])
async def get_questions_by_bank(bank_id: int, limit: int = 100):
    """获取题库下的题目列表"""
    try:
        return QuestionService().get_questions_by_bank(bank_id, limit)
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.post("/move-to-bank/")
async def move_questions_to_bank(question_ids: List[int], target_bank_id: int):
    """将题目移动到指定题库"""
    try:
        QuestionService().move_questions_to_bank(question_ids, target_bank_id)
        return {"message": "Questions moved successfully"}
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.post("/bank/{bank_id}/permissions",
             response_model=QuestionBankPermission)
async def add_bank_permission(bank_id: int, user_id: int, permission_type: str,
                              granted_by: int):
    """添加题库权限"""
    try:
        return QuestionService().add_bank_permission(bank_id, user_id,
                                                     permission_type,
                                                     granted_by)
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.delete("/bank/permissions/{permission_id}")
async def remove_bank_permission(permission_id: int):
    """移除题库权限"""
    try:
        QuestionService().remove_bank_permission(permission_id)
        return {"message": "Permission removed successfully"}
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.get("/bank/{bank_id}/permissions",
            response_model=List[QuestionBankPermission])
async def get_bank_permissions(bank_id: int):
    """获取题库的所有权限"""
    try:
        return QuestionService().get_bank_permissions(bank_id)
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.get("/bank/{bank_id}/permissions/{user_id}",
            response_model=List[QuestionBankPermission])
async def get_user_bank_permissions(bank_id: int, user_id: int):
    """获取用户对指定题库的权限"""
    try:
        return QuestionService().get_user_bank_permissions(bank_id, user_id)
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.get("/bank/{bank_id}/statistics")
async def get_bank_statistics(bank_id: int):
    """获取题库统计信息"""
    try:
        return QuestionService().get_bank_statistics(bank_id)
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.post("/bank/{bank_id}/validate-access")
async def validate_bank_access(bank_id: int, user_id: int):
    """验证用户是否有访问题库的权限"""
    try:
        QuestionService().validate_bank_access(bank_id, user_id)
        return {"message": "Access granted"}
    except Exception as e:
        raise HTTPException(status_code=403, detail=str(e))


@router.post("/bank/{bank_id}/update-statistics")
async def update_bank_statistics(bank_id: int):
    """更新题库统计信息"""
    try:
        return QuestionService().update_bank_statistics(bank_id)
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.post("/questions/{question_id}/knowledge_points")
async def add_knowledge_point(question_id: int, knowledge_point_id: int):
    """为题目添加知识点"""
    try:
        return QuestionService().add_knowledge_point(question_id,
                                                     knowledge_point_id)
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.delete(
    "/questions/{question_id}/knowledge_points/{knowledge_point_id}")
async def remove_knowledge_point(question_id: int, knowledge_point_id: int):
    """移除题目的知识点"""
    try:
        return QuestionService().remove_knowledge_point(
            question_id, knowledge_point_id)
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.get("/questions/{question_id}/knowledge_points")
async def get_question_knowledge_points(question_id: int):
    """获取题目的知识点列表"""
    try:
        return QuestionService().get_question_knowledge_points(question_id)
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))
