from fastapi import APIRouter, Depends, HTTPException, status
from typing import List
from ..services.user_service import UserService
from ..models.user import UserCreate, UserUpdate, UserResponse
from ..middleware.auth_middleware import get_current_user, require_admin

router = APIRouter(prefix="/users", tags=["Users"])


@router.post("/",
             response_model=UserResponse,
             status_code=status.HTTP_201_CREATED)
async def create_user(user: UserCreate, _=Depends(require_admin)):
    return await UserService.create_user(user)


@router.get("/", response_model=List[UserResponse])
async def get_users(skip: int = 0, limit: int = 100, _=Depends(require_admin)):
    return await UserService.get_users(skip, limit)


@router.get("/{user_id}", response_model=UserResponse)
async def get_user(user_id: int,
                   current_user: dict = Depends(get_current_user)):
    if current_user["id"] != user_id and not current_user["is_admin"]:
        raise HTTPException(status_code=status.HTTP_403_FORBIDDEN,
                            detail="Not authorized to access this user")
    user = await UserService.get_user(user_id)
    if not user:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND,
                            detail="User not found")
    return user


@router.put("/{user_id}", response_model=UserResponse)
async def update_user(user_id: int,
                      user: UserUpdate,
                      current_user: dict = Depends(get_current_user)):
    if current_user["id"] != user_id and not current_user["is_admin"]:
        raise HTTPException(status_code=status.HTTP_403_FORBIDDEN,
                            detail="Not authorized to update this user")
    updated_user = await UserService.update_user(user_id, user)
    if not updated_user:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND,
                            detail="User not found")
    return updated_user


@router.delete("/{user_id}")
async def delete_user(user_id: int, _=Depends(require_admin)):
    success = await UserService.delete_user(user_id)
    if not success:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND,
                            detail="User not found")
    return {"message": "User deleted successfully"}
