import logging, os
import logging.handlers
from typing import Optional
from ..config.config_loader import config


# 初始化日志系统
def setup_logging():
    """初始化日志配置"""
    logging_config = {
        "level": config.app.log_level,
        "console": True,
        "format":
        "%(asctime)s %(levelname)s %(name)s:%(lineno)d - %(message)s",
        "datefmt": "%Y-%m-%d %H:%M:%S",
        "file":
        config.logging.get("file") if hasattr(config, "logging") else None
    }

    logger = logging.getLogger("app")
    _configure_logger(logger, logging_config)
    return logger


def _configure_logger(logger: logging.Logger, config: dict) -> None:
    """根据配置初始化日志记录器"""
    level = config.get("level", "INFO")
    format = config.get(
        "format",
        "%(asctime)s %(levelname)s %(name)s:%(lineno)d - %(message)s")
    datefmt = config.get("datefmt", "%Y-%m-%d %H:%M:%S")

    logger.setLevel(level)
    formatter = logging.Formatter(format, datefmt)

    # 控制台输出
    if config.get("console", True):
        console_handler = logging.StreamHandler()
        console_handler.setFormatter(formatter)
        logger.addHandler(console_handler)

    # 文件输出
    if config.get("file"):
        # 将相对路径转换为绝对路径
        log_file = os.path.join(
            os.path.dirname(
                os.path.dirname(os.path.dirname(os.path.dirname(__file__)))),
            config["file"])

        # 确保日志目录存在
        log_dir = os.path.dirname(log_file)
        if not os.path.exists(log_dir):
            os.makedirs(log_dir, exist_ok=True)

        file_handler = logging.handlers.TimedRotatingFileHandler(
            log_file,
            when="midnight",
            interval=1,
            backupCount=7,
            encoding="utf-8")
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)


# 导出常用日志函数
def debug(msg: str, *args, **kwargs) -> None:
    """记录调试信息"""
    logging.getLogger("app").debug(msg, *args, **kwargs)


def info(msg: str, *args, **kwargs) -> None:
    """记录一般信息"""
    logging.getLogger("app").info(msg, *args, **kwargs)


def warning(msg: str, *args, **kwargs) -> None:
    """记录警告信息"""
    logging.getLogger("app").warning(msg, *args, **kwargs)


def error(msg: str, *args, **kwargs) -> None:
    """记录错误信息"""
    logging.getLogger("app").error(msg, *args, **kwargs)


def exception(msg: str, *args, **kwargs) -> None:
    """记录异常信息"""
    logging.getLogger("app").exception(msg, *args, **kwargs)


def get_logger() -> logging.Logger:
    """获取日志记录器"""
    return logging.getLogger("app")
