from typing import List, Optional
from sqlalchemy import Column, String, Text, ForeignKey, JSON
from sqlalchemy.orm import relationship
from .base_model import BaseModel
from .question_type import QuestionType
from .question_difficulty import QuestionDifficulty
from .question_category import QuestionCategory
from .question_tag import QuestionTag
from .question_option import QuestionOption
from .question_analysis import QuestionAnalysis
from .question_feedback import QuestionFeedback
from .question_comment import QuestionComment
from .question_attachment import QuestionAttachment
from .question_version import QuestionVersion
from .question_relation import QuestionRelation


class Question(BaseModel):
    """题目模型"""
    __tablename__ = "question"

    title = Column(String(255), nullable=False, comment="题目标题")
    content = Column(Text, nullable=False, comment="题目内容")
    type_id = Column(Integer,
                     ForeignKey("questiontype.id"),
                     nullable=False,
                     comment="题目类型ID")
    difficulty_id = Column(Integer,
                           ForeignKey("questiondifficulty.id"),
                           nullable=False,
                           comment="难度等级ID")
    category_id = Column(Integer,
                         ForeignKey("questioncategory.id"),
                         nullable=True,
                         comment="分类ID")
    tags = Column(JSON, nullable=True, comment="标签列表")
    answer = Column(Text, nullable=True, comment="题目答案")
    explanation = Column(Text, nullable=True, comment="题目解析")
    status = Column(Integer, default=1, comment="题目状态 1:正常 0:禁用")
    is_public = Column(Integer, default=0, comment="是否公开 1:是 0:否")
    creator_id = Column(Integer, nullable=False, comment="创建者ID")
    reviewer_id = Column(Integer, nullable=True, comment="审核者ID")
    review_status = Column(Integer,
                           default=0,
                           comment="审核状态 0:待审核 1:审核通过 2:审核不通过")
    review_comment = Column(Text, nullable=True, comment="审核意见")
    source = Column(String(255), nullable=True, comment="题目来源")
    extra_data = Column(JSON, nullable=True, comment="扩展数据")

    # 关系定义
    type = relationship("QuestionType", back_populates="questions")
    difficulty = relationship("QuestionDifficulty", back_populates="questions")
    category = relationship("QuestionCategory", back_populates="questions")
    options = relationship("QuestionOption", back_populates="question")
    analyses = relationship("QuestionAnalysis", back_populates="question")
    feedbacks = relationship("QuestionFeedback", back_populates="question")
    comments = relationship("QuestionComment", back_populates="question")
    attachments = relationship("QuestionAttachment", back_populates="question")
    versions = relationship("QuestionVersion", back_populates="question")
    relations = relationship("QuestionRelation", back_populates="question")
    knowledge_points = relationship("KnowledgePoint",
                                    secondary="question_knowledge_point",
                                    back_populates="questions")

    def __repr__(self):
        return f"<Question(id={self.id}, title={self.title})>"

    @classmethod
    def create_with_options(cls, db: Session, title: str, content: str,
                            type_id: int, difficulty_id: int,
                            options: List[Dict[str,
                                               Any]], **kwargs) -> "Question":
        """创建题目并添加选项"""
        question = cls.create(db,
                              title=title,
                              content=content,
                              type_id=type_id,
                              difficulty_id=difficulty_id,
                              **kwargs)
        for option_data in options:
            QuestionOption.create(db, question_id=question.id, **option_data)
        return question

    @classmethod
    def get_with_details(cls, db: Session, id: int) -> Optional["Question"]:
        """获取题目详情"""
        return (db.query(cls).options(
            joinedload(cls.type),
            joinedload(cls.difficulty),
            joinedload(cls.category),
            joinedload(cls.options),
            joinedload(cls.analyses),
            joinedload(cls.feedbacks),
            joinedload(cls.comments),
            joinedload(cls.attachments),
            joinedload(cls.versions),
            joinedload(cls.relations),
        ).filter(cls.id == id).first())
