import hashlib
import os
import base64
from typing import Optional
from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC


class CryptoUtil:
    """加密工具类"""

    @staticmethod
    def generate_salt(length: int = 16) -> bytes:
        """生成随机盐值
        
        Args:
            length: 盐值长度
            
        Returns:
            随机生成的盐值
        """
        return os.urandom(length)

    @staticmethod
    def hash_password(password: str, salt: Optional[bytes] = None) -> str:
        """哈希密码
        
        Args:
            password: 原始密码
            salt: 盐值
            
        Returns:
            哈希后的密码字符串
        """
        if salt is None:
            salt = CryptoUtil.generate_salt()

        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(),
            length=32,
            salt=salt,
            iterations=100000,
        )
        key = base64.urlsafe_b64encode(kdf.derive(password.encode()))
        return key.decode()

    @staticmethod
    def md5(data: str) -> str:
        """计算MD5哈希值
        
        Args:
            data: 输入数据
            
        Returns:
            MD5哈希值
        """
        return hashlib.md5(data.encode()).hexdigest()

    @staticmethod
    def sha256(data: str) -> str:
        """计算SHA256哈希值
        
        Args:
            data: 输入数据
            
        Returns:
            SHA256哈希值
        """
        return hashlib.sha256(data.encode()).hexdigest()

    @staticmethod
    def encrypt(data: str, key: str) -> str:
        """对称加密
        
        Args:
            data: 要加密的数据
            key: 加密密钥
            
        Returns:
            加密后的字符串
        """
        f = Fernet(key.encode())
        return f.encrypt(data.encode()).decode()

    @staticmethod
    def decrypt(encrypted_data: str, key: str) -> str:
        """对称解密
        
        Args:
            encrypted_data: 加密后的数据
            key: 解密密钥
            
        Returns:
            解密后的字符串
        """
        f = Fernet(key.encode())
        return f.decrypt(encrypted_data.encode()).decode()

    @staticmethod
    def generate_key() -> str:
        """生成加密密钥
        
        Returns:
            生成的密钥字符串
        """
        return Fernet.generate_key().decode()
