import os
import shutil
from typing import Optional, Union
from pathlib import Path


class FileUtil:
    """文件操作工具类"""

    @staticmethod
    def read_file(path: Union[str, Path]) -> Optional[str]:
        """读取文件内容
        
        Args:
            path: 文件路径
            
        Returns:
            文件内容字符串，如果文件不存在返回None
        """
        if not os.path.exists(path):
            return None
        with open(path, 'r', encoding='utf-8') as f:
            return f.read()

    @staticmethod
    def write_file(path: Union[str, Path], content: str) -> bool:
        """写入文件内容
        
        Args:
            path: 文件路径
            content: 要写入的内容
            
        Returns:
            bool: 是否写入成功
        """
        try:
            with open(path, 'w', encoding='utf-8') as f:
                f.write(content)
            return True
        except Exception:
            return False

    @staticmethod
    def ensure_dir_exists(path: Union[str, Path]) -> bool:
        """确保目录存在
        
        Args:
            path: 目录路径
            
        Returns:
            bool: 是否成功创建目录
        """
        try:
            os.makedirs(path, exist_ok=True)
            return True
        except Exception:
            return False

    @staticmethod
    def delete_file(path: Union[str, Path]) -> bool:
        """删除文件
        
        Args:
            path: 文件路径
            
        Returns:
            bool: 是否删除成功
        """
        try:
            if os.path.exists(path):
                os.remove(path)
                return True
            return False
        except Exception:
            return False

    @staticmethod
    def copy_file(src: Union[str, Path], dst: Union[str, Path]) -> bool:
        """复制文件
        
        Args:
            src: 源文件路径
            dst: 目标文件路径
            
        Returns:
            bool: 是否复制成功
        """
        try:
            shutil.copy(src, dst)
            return True
        except Exception:
            return False

    @staticmethod
    def get_file_size(path: Union[str, Path]) -> Optional[int]:
        """获取文件大小
        
        Args:
            path: 文件路径
            
        Returns:
            文件大小（字节），如果文件不存在返回None
        """
        if not os.path.exists(path):
            return None
        return os.path.getsize(path)

    @staticmethod
    def get_file_extension(path: Union[str, Path]) -> str:
        """获取文件扩展名
        
        Args:
            path: 文件路径
            
        Returns:
            文件扩展名（小写）
        """
        return Path(path).suffix.lower()

    @staticmethod
    def is_file(path: Union[str, Path]) -> bool:
        """判断路径是否为文件
        
        Args:
            path: 路径
            
        Returns:
            bool: 是否为文件
        """
        return os.path.isfile(path)

    @staticmethod
    def is_dir(path: Union[str, Path]) -> bool:
        """判断路径是否为目录
        
        Args:
            path: 路径
            
        Returns:
            bool: 是否为目录
        """
        return os.path.isdir(path)
