import requests
import socket
import urllib.parse
from typing import Optional, Dict, Any
from urllib.parse import urlparse


class NetworkUtil:
    """网络工具类"""

    @staticmethod
    def get(url: str,
            params: Optional[Dict] = None,
            headers: Optional[Dict] = None) -> Optional[Dict]:
        """发送GET请求
        
        Args:
            url: 请求URL
            params: 请求参数
            headers: 请求头
            
        Returns:
            响应数据字典，如果请求失败返回None
        """
        try:
            response = requests.get(url, params=params, headers=headers)
            response.raise_for_status()
            return response.json()
        except Exception:
            return None

    @staticmethod
    def post(url: str,
             data: Optional[Dict] = None,
             headers: Optional[Dict] = None) -> Optional[Dict]:
        """发送POST请求
        
        Args:
            url: 请求URL
            data: 请求体数据
            headers: 请求头
            
        Returns:
            响应数据字典，如果请求失败返回None
        """
        try:
            response = requests.post(url, json=data, headers=headers)
            response.raise_for_status()
            return response.json()
        except Exception:
            return None

    @staticmethod
    def is_valid_ip(ip: str) -> bool:
        """验证IP地址是否有效
        
        Args:
            ip: IP地址字符串
            
        Returns:
            bool: 是否为有效IP地址
        """
        try:
            socket.inet_aton(ip)
            return True
        except socket.error:
            return False

    @staticmethod
    def get_domain(url: str) -> Optional[str]:
        """从URL中提取域名
        
        Args:
            url: URL字符串
            
        Returns:
            域名，如果URL无效返回None
        """
        try:
            parsed = urlparse(url)
            return parsed.netloc
        except Exception:
            return None

    @staticmethod
    def is_online() -> bool:
        """检查网络是否在线
        
        Returns:
            bool: 是否在线
        """
        try:
            socket.create_connection(("www.baidu.com", 80))
            return True
        except OSError:
            return False

    @staticmethod
    def build_url(base: str, params: Dict[str, Any]) -> str:
        """构建带参数的URL
        
        Args:
            base: 基础URL
            params: 参数字典
            
        Returns:
            带参数的完整URL
        """
        return f"{base}?{urllib.parse.urlencode(params)}"
